package com.hzya.frame.plugin.oa.payment.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.cbs8.dto.req.PayResultRequestDTO;
import com.hzya.frame.cbs8.dto.res.PayResponseDTO;
import com.hzya.frame.cbs8.dto.res.PayResultResDTO;
import com.hzya.frame.cbs8.service.ICbs8Service;
import com.hzya.frame.cbs8.util.CBSUtil;
import com.hzya.frame.cbs8.util.PayState;
import com.hzya.frame.plugin.oa.payment.service.IPaymentPluginService;
import com.hzya.frame.seeyon.cbs8.entity.CbsLogEntity;
import com.hzya.frame.seeyon.cbs8.entity.PaymentEntity;
import com.hzya.frame.seeyon.cbs8.service.ICbsLogService;
import com.hzya.frame.seeyon.cbs8.service.IPaymentService;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * @Description 支付申请
 * @Author xiangerlin
 * @Date 2025/5/29 15:43
 **/
public class PaymentPluginServiceImpl implements IPaymentPluginService {

    Logger logger = LoggerFactory.getLogger(PaymentPluginServiceImpl.class);
    @Autowired
    private IPaymentService paymentService;
    @Autowired
    private ICbs8Service cbs8Service;
    @Autowired
    private ICbsLogService cbsLogService;;
    /**
     * 支付申请
     *
     * @param requestJson
     * @return
     */
    @Override
    public JsonResultEntity payRequest(JSONObject requestJson) {
        String headersStr = requestJson.getString("headers");//请求头
        String formAppId = requestJson.getString("formAppId");
        String eventType = requestJson.getString("eventType");
        String id = requestJson.getString("id");
        String dataSouceCode = requestJson.getString("sourceCode");
        Assert.notEmpty(id,"id不能为空");
        Assert.notEmpty(formAppId,"formAppId不能为空");
        Assert.notEmpty(eventType,"eventType不能为空");
        Assert.notEmpty(headersStr,"headers不能为空");
        Assert.notEmpty(dataSouceCode,"数据源编码不能为空");
        logger.info("准备开始查询待支付的单据");
        PaymentEntity paymentEntity = new PaymentEntity();
        paymentEntity.setId(id);
        paymentEntity.setDataSourceCode(dataSouceCode);
        try {
            //查询OA待支付的数据
            List<PaymentEntity> paymentList = paymentService.queryUnpaid(paymentEntity);
            if (CollectionUtils.isNotEmpty(paymentList) && paymentList.size() == 1){
                PaymentEntity pay = paymentList.get(0);
                PayResponseDTO payResponseDTO = cbs8Service.payApply(JSONObject.toJSONString(pay));
                String payResStr = JSONObject.toJSONString(payResponseDTO);
                logger.info("支付申请响应结果:{}",payResStr);
                if (null != payResponseDTO){
                    //保存支付日志
                    saveCbsLog(pay,payResponseDTO,dataSouceCode);
                    if (payResponseDTO.getSuccessed()) {
                        return BaseResult.getSuccessMessageEntity("支付申请成功",payResponseDTO.getBusNum());
                    }else {
                        BaseResult.getFailureMessageEntity("支付申请失败：{}",payResStr);
                    }
                }else {
                    BaseResult.getFailureMessageEntity("支付申请失败");
                }
                return BaseResult.getSuccessMessageEntity("",payResponseDTO);
            }
        }catch (Exception e){
            logger.error("OA单据推CBS支付申请失败:{}",e);
        }

        return null;
    }

    /**
     * 查询交易结果
     *
     * @param requestJson
     */
    @Override
    public void queryPayResult(JSONObject requestJson) {
        String dataSourceCode = requestJson.getString("sourceCode");
        Assert.notEmpty(dataSourceCode,"数据源编码不能为空");
        try {
            CbsLogEntity cbsLogEntity = new CbsLogEntity();
            cbsLogEntity.setDataSourceCode(dataSourceCode);
            List<CbsLogEntity> queryList = cbsLogService.queryInPayment(cbsLogEntity);
            if (CollectionUtils.isNotEmpty(queryList)){
                for (CbsLogEntity entity : queryList) {
                    List<PayResultResDTO> payResultResDTOList = cbs8Service.queryPayResult(new PayResultRequestDTO(entity.getBill_code()));
                    if (CollectionUtils.isNotEmpty(payResultResDTOList)){
                        PayResultResDTO payResultResDTO = payResultResDTOList.get(0);
                        String status = payResultResDTO.getStatus();//支付申请状态
                        String pay_status = payResultResDTO.getPayStatus();//支付状态
                        if (!PayState.p.getType().equals(pay_status)){//不等于支付中的时候才更新
                            //如果支付状态为空，保存支付申请状态，如果支付状态不为空，则保存支付状态
                            PaymentEntity paymentEntity = new PaymentEntity();
                            paymentEntity.setOaId(entity.getOa_id());
                            paymentEntity.setDataSourceCode(dataSourceCode);
                            List<PaymentEntity> paymentList = paymentService.query(paymentEntity);
                            if (CollectionUtils.isNotEmpty(paymentList)){
                                paymentEntity = paymentList.get(0);
                                if (StrUtil.isEmpty(pay_status)) {
                                    paymentEntity.setPayResult(PayState.payStateGetValue(status));//支付申请状态 支付状态和支付申请状态用一个
                                } else {
                                    paymentEntity.setPayResult(PayState.payStateGetValue(pay_status));//支付状态 支付状态和支付申请状态用一个
                                }
                                if (StrUtil.isNotEmpty(pay_status) && pay_status.equals(PayState.g.getType())) {
                                    paymentEntity.setPayDate(CBSUtil.convertTimestampToString(payResultResDTO.getPayDate()));//支付时间
                                }
                                paymentService.updatePayState(paymentEntity);
                                //更新日志表状态
                                entity.setPay_state(paymentEntity.getPayResult());
                                entity.setApply_state(PayState.payStateGetValue(status));
                                entity.setDataSourceCode(dataSourceCode);
                                cbsLogService.update(entity);
                            }
                        }
                    }
                }
            }
        }catch (Exception e){
            logger.error("查询交易结果出错:{}",e);
        }
    }

    /**
     * 保存支付日志
     * @param entity 支付参数
     * @param payResponseDTO cbs返回参数
     * @param dataSourceCode 数据源编码
     * @throws Exception
     */
    private void saveCbsLog(PaymentEntity entity,PayResponseDTO payResponseDTO,String dataSourceCode)throws Exception{
        if (null != payResponseDTO){
            //4. 保存日志
            CbsLogEntity cbsLogEntity = new CbsLogEntity();
            cbsLogEntity.setTitle(entity.getTitle());
            cbsLogEntity.setPay_company(entity.getPayCompany());
            cbsLogEntity.setPayee(entity.getRevAccountName());
            cbsLogEntity.setAmount(entity.getAmount());
            cbsLogEntity.setOa_id(entity.getOaId());
            cbsLogEntity.setBill_code(Convert.toStr(entity.getReferenceNumNew(),entity.getReferenceNum()));
            cbsLogEntity.setTab_name_ch(entity.getBillName());
            cbsLogEntity.setTab_name_en(entity.getTableName());
            Boolean successed = payResponseDTO.getSuccessed();
            if (successed){
                cbsLogEntity.setPay_state(PayState.p.getValue());
                cbsLogEntity.setApply_state(PayState.two.getValue());
                cbsLogEntity.setCbs_apply_code(payResponseDTO.getBusNum());
                cbsLogEntity.setSuccessed("true");
                entity.setPayResult(PayState.p.getValue());
            }else {
                cbsLogEntity.setPay_state("推送失败");
                cbsLogEntity.setMessage(payResponseDTO.getErrorMsg());
                cbsLogEntity.setSuccessed("false");
                entity.setPayResult("推送失败");
            }
            cbsLogEntity.setDataSourceCode(dataSourceCode);
            cbsLogService.saveLog(cbsLogEntity);
            //5.更新视图的支付状态
            paymentService.updatePayState(entity);
        }
    }
}
