package com.hzya.frame.plugin.oa.poorder.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.execsql.entity.ExecSqlEntity;
import com.hzya.frame.execsql.service.IExecSqlService;
import com.hzya.frame.plugin.oa.poorder.entity.PoOrderEntity;
import com.hzya.frame.plugin.oa.poorder.service.IPoOrderExtService;
import com.hzya.frame.plugin.oa.poorder.service.IPoOrderService;
import com.hzya.frame.seeyon.enums.ColEventTypeEnum;
import com.hzya.frame.seeyon.org.member.entity.UserViewEntity;
import com.hzya.frame.seeyon.org.member.service.IUserViewService;
import com.hzya.frame.seeyon.util.OARestUtil;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import java.util.LinkedList;
import java.util.List;

/**
 * @Description 接收NCC采购订单的请求
 * @Author xiangerlin
 * @Date 2025/6/12 17:04
 **/
@Service(value="nCCPoOrderServiceImpl")
public class PoOrderServiceImpl implements IPoOrderService {
    Logger logger = LoggerFactory.getLogger(PoOrderServiceImpl.class);
    @Autowired
    private IExecSqlService execSqlService;
    @Autowired
    private OARestUtil oaRestUtil;
    @Autowired
    private IUserViewService userViewService;
    @Autowired
    private IPoOrderExtService poOrderExtService;
    @Value("${zt.url}")
    private String interfaceUrl;
    @Value("${oa.po_order_template_code:}")
    private String templateCode;
    /**
     * 接收NCC的请求，并转换成OA的参数
     *
     * @param reqJson
     * @return
     */
    @Override
    public JsonResultEntity buildOAParam(JSONObject reqJson) {
        JSONObject nccObj = JSONObject.parseObject(reqJson.getString("jsonStr"));
        logger.info("中台接收到NCC采购订单的请求参数:{}",reqJson.getString("jsonStr"));
        Assert.notNull(nccObj,"参数不能为空");
        JSONObject head = nccObj.getJSONObject("head");

        JSONArray detailList = nccObj.getJSONArray("body");
        Assert.notNull(head,"参数不能为空");
        Assert.notNull(detailList,"参数不能为空");
        UserViewEntity userView = new UserViewEntity();
        userView.setStaffNumber(head.getString("user_code"));
        userView.setDataSourceCode("JH-OA");
        UserViewEntity userViewEntity = userViewService.queryByStaffNumber(userView);
        //组装参数
        JSONObject formmain_0256 = new JSONObject();//主表数据
        List<JSONObject> formson_0257_list = new LinkedList<>();//子表数据
        formmain_0256.put("采购组织名称",head.getString("org_name"));
        formmain_0256.put("订单日期",head.getString("dbilldate"));
        formmain_0256.put("供应商",head.getString("sup_name"));
        formmain_0256.put("采购员姓名",head.getString("cemployeename"));
        formmain_0256.put("供应商名称",head.getString("sup_name"));
        formmain_0256.put("开票供应商",head.getString("invc_sup_name"));
        formmain_0256.put("付款协议",head.getString("payterm_name"));
        formmain_0256.put("单据状态",head.getString("forderstatus"));
        formmain_0256.put("备注",head.getString("vmemo"));
        formmain_0256.put("补货",booleanToStr(head.getString("bisreplenish")));
        formmain_0256.put("退货",booleanToStr(head.getString("breturn")));
        formmain_0256.put("总数量",head.getString("ntotalastnum"));
        formmain_0256.put("总价税合计",head.getString("ntotalorigmny"));
        formmain_0256.put("币种",head.getString("currtype_name"));
        formmain_0256.put("退货退库基于原订单补货",booleanToStr(head.getString("brefwhenreturn")));
        formmain_0256.put("最终关闭",booleanToStr(head.getString("bfinalclose")));
        formmain_0256.put("采购部门",head.getString("dept_name"));
        formmain_0256.put("订单类型",head.getString("billtypename"));
        formmain_0256.put("NCC采购订单编号",head.getString("vbillcode"));
        formmain_0256.put("NCC采购订单PK",head.getString("pk_order"));
        //子表
        for (int i=0; i < detailList.size(); i++){
            JSONObject detail = (JSONObject) detailList.get(i);
            JSONObject formson_0257 = new JSONObject();
            formson_0257.put("收货库存组织",detail.getString("arrvstoorg_name").toLowerCase());
            formson_0257.put("行号",detail.getString("crowno"));
            formson_0257.put("物料编码",detail.getString("material_code"));
            formson_0257.put("物料名称",detail.getString("material_name"));
            formson_0257.put("规格",detail.getString("materialspec"));
            formson_0257.put("型号",detail.getString("materialtype"));
            formson_0257.put("单位",detail.getString("cunitname"));
            formson_0257.put("数量",detail.getString("nnum"));
            formson_0257.put("换算率",detail.getString("vchangerate"));
            formson_0257.put("无税单价",detail.getString("nqtorigprice"));
            formson_0257.put("含税单价",detail.getString("nqtorigtaxprice"));
            formson_0257.put("价税合计",detail.getString("norigtaxmny"));
            formson_0257.put("集团本币无税金额",detail.getString("norigmny"));
            formson_0257.put("集团本币价税合计",detail.getString("norigtaxmny"));
            formson_0257.put("计划收货日期",detail.getString("dplanarrvdate"));
            formson_0257.put("收货仓库",detail.getString("recvstordoc_name"));
            formson_0257.put("税率",detail.getString("ntaxrate"));
            formson_0257.put("赠品",booleanToStr(detail.getString("blargess")));
            formson_0257.put("NCC采购订单表体PK",detail.getString("pk_order_b"));
            formson_0257_list.add(formson_0257);
        }
        String param = oaRestUtil.processParamValueOf(formmain_0256, formson_0257_list, templateCode, "formmain_0256", "formson_0257");
        String token = oaRestUtil.getToken(userViewEntity.getLoginName(), "8000640001");
        logger.info("NCC发起OA采购订单请求参数{}",param);
        //调用OA接口
        String result = HttpRequest.post(interfaceUrl)
                .header("token",token)// token
                .header("appId","800064")// 建辉OA应用
                .header("apiCode","8000640000")//流程表单接口
                .header("publicKey","ZJYAAr9TeFduBYu7uJeie2KYdQsnBaEYZHmOjPWMMVZmzelCe7eDIk+3zDUT+v578prj")//NCC应用key
                .header("secretKey","f4SSnYcDQmdkwwuGJd2+934q6lM1NnUm5dmOw/4Wvy2mo6PgkKsI/drXYyyLWQCaj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")//NCC应用密钥
                .body(param).execute().body();
        logger.info("NCC发起OA采购订单响应参数:{}",result);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (null == resultJson){
            return BaseResult.getFailureMessageEntity("发起OA审批失败");
        }
        if (!resultJson.getBoolean("flag")){
            return BaseResult.getFailureMessageEntity("发起OA审批失败"+resultJson.getString("msg"));
        }
        return BaseResult.getSuccessMessageEntity(resultJson.getJSONObject("attribute"));
    }


    /**
     * OA审批事件
     *POSTQUANTITY
     * @param requestJson
     * @return
     */
    @Override
    public JsonResultEntity approvedEvent(JSONObject requestJson)throws Exception {
        //数据源编码
        String datasourceCode = requestJson.getString("sourceCode");
        String task_living_details_id = requestJson.getString("integration_task_living_details_id");
        String headersStr = requestJson.getString("headers");//请求头
        String eventType = requestJson.getString("eventType");
        String summaryId = requestJson.getString("summaryId");
        JSONObject headers = requestJson.getJSONObject("headers");
        Assert.notEmpty(eventType,"eventType不能为空");
        Assert.notEmpty(headersStr,"headers不能为空");
        String formmainTableName = headers.getString("formmainTableName");
        JSONObject jsonStrObj = requestJson.getJSONObject("jsonStr");
        JSONObject businessData = jsonStrObj.getJSONObject("businessDataStr");
        JSONObject formmainData = businessData.getJSONObject(formmainTableName);
        if (null != formmainData){
            String pk_order = formmainData.getString("field0056");//采购订单pk
            String vbillcode = formmainData.getString("field0055");//采购订单code
            String userId = headers.getString("user_id");//审批人id
            JSONObject json = new JSONObject();
            json.put("pk_order",pk_order);
            json.put("vbillcode",vbillcode);
            json.put("user_id",userId);
            String params = json.toString();
            //组装参数，调用NCC审批接口
            logger.info("OA采购订单审批后传到NCC请购参数。:{},{}",eventType,params);
            String apiCode = "";
            if (ColEventTypeEnum.ONPROCESSFINISHED.getType().equals(eventType)){//审批完成
                apiCode = "8000650002";
            }else if (ColEventTypeEnum.ONSTOP.getType().equals(eventType) || ColEventTypeEnum.ONCANCEL.getType().equals(eventType)){//流程终止/撤销
                apiCode = "8000650003";
            }
            String result = HttpRequest.post(interfaceUrl)
                    .header("appId", "800065")//NCC应用
                    .header("apiCode", apiCode)//NCC采购订单审批接口
                    .header("publicKey", "ZJYAbkr9+XjnDrlfQCRKXtpVvg/BjxqtxzcLgg5TIGagEKJCe7eDIk+3zDUT+v578prj")//OA公钥
                    .header("secretKey", "2GR4+yrcx+Ev+pN0Q6V6wxCdvisPX7wzNKBgc5SsIYGxYI5GTISXT6GvTPfp1u2Rj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")//OA密钥
                    .body(params)//表单内容
                    .timeout(30000)//超时，毫秒
                    .execute().body();
            logger.info("======OA采购单审批后传到NCC响应参数:{}======",result);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (null == resultJson){
                return BaseResult.getFailureMessageEntity("审批NCC采购订单失败");
            }
            if (!resultJson.getBoolean("flag")){
                return BaseResult.getFailureMessageEntity("审批NCC采购订单失败"+resultJson.getString("msg"));
            }
            //如果是流程终止或撤销，更新OA校验字段
            if (ColEventTypeEnum.ONSTOP.getType().equals(eventType) || ColEventTypeEnum.ONCANCEL.getType().equals(eventType)){
                String id = formmainData.getString("id");
                if (StrUtil.isNotEmpty(id)){
                    logger.info("准备更新OA采购订单校验字段");
                    PoOrderEntity poOrderEntity = new PoOrderEntity();
                    poOrderEntity.setId(id);
                    poOrderEntity.setDataSourceCode("JH-OA");
                    poOrderEntity.setField0058("-1");
                    poOrderEntity.setField0059("-1");
                    poOrderExtService.updateVerifyFlag(poOrderEntity);
                }
            }
            return BaseResult.getSuccessMessageEntity(resultJson.getJSONObject("attribute"));
        }
        return BaseResult.getSuccessMessageEntity("审批NCC采购订单");
    }
    /**
     * 把NCC的 boolean类型转成OA
     * NCC Y/N
     * OA 1/0
     * @param nccBoolean
     * @return
     */
    private String booleanToStr(String nccBoolean){
        if ("Y".equals(nccBoolean)){
            return "1";
        }
        return "0";
    }

}
