package com.hzya.frame.ncc.v202005.service.impl;


import com.hzya.frame.ncc.v202005.service.INccTokenService;
import com.hzya.frame.sysnew.application.entity.SysExtensionApiEntity;
import com.hzya.frame.util.bipV3.Encryption;
import com.hzya.frame.util.bipV3.SHA256Util;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.net.URLEncoder;
import java.util.*;


@Service(value="nccTokenService")
public class NccTokenServiceImpl implements INccTokenService {
	private final Logger logger = LoggerFactory.getLogger(this.getClass());


	@Override
	public SysExtensionApiEntity getNccToken(SysExtensionApiEntity entity) {
		try {
			Map<String, String> headers = entity.getHeaders();
			String client_id = headers.get("client_id");
			Map<String, String> paramMap = new HashMap<String, String>();
			// 密码模式认证
			paramMap.put("grant_type", "client_credentials");
			// 第三方应用id
			paramMap.put("client_id", client_id);
			// 第三方应用secret]
			String client_secret =  headers.get("client_secret");
			// 账套编码
			String busi_center =  headers.get("busi_center");
			//用户编码
			String usercode =  headers.get("usercode");
			//数据库编码
			String dsname =  headers.get("dsname");
			String pubKey =  headers.get("pubKey");
			// 第三方应用secret 公钥加密]
			paramMap.put("client_secret", URLEncoder.encode(Encryption.pubEncrypt(pubKey, client_secret), "utf-8"));
			// 账套编码
			paramMap.put("biz_center", busi_center);
			// // TODO 传递数据源和ncc登录用户
			paramMap.put("dsname", dsname);
			paramMap.put("usercode", usercode);
			// 签名
			String sign = SHA256Util.getSHA256(client_id + client_secret + pubKey,pubKey);
			paramMap.put("signature", sign);
			StringBuffer querys = new StringBuffer();
			querys.append("biz_center="+busi_center);
			querys.append("&grant_type="+"client_credentials");
			querys.append("&signature="+sign);
			querys.append("&dsname="+dsname);
			querys.append("&client_secret="+ URLEncoder.encode(Encryption.pubEncrypt(pubKey, client_secret), "utf-8"));
			querys.append("&usercode="+usercode);
			querys.append("&client_id="+client_id);
			entity.setQuerys(querys.toString());
			headers.put("content-type","application/x-www-form-urlencoded");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return entity;
	}



	@Override
	public SysExtensionApiEntity getNccBill(SysExtensionApiEntity entity) {
		try {
			Map<String, String> headers = entity.getHeaders();
			String client_id = headers.get("client_id");
			//token信息
			String access_token =  headers.get("access_token");
			//公钥
			String pubKey =  headers.get("pubKey");
			StringBuffer sb = new StringBuffer();
			sb.append(client_id);
			String parm = entity.getBodys();
			if (StringUtils.isNotBlank(parm)) {
				sb.append(parm);
			}
			sb.append(pubKey);
			// 签名
			String sign = SHA256Util.getSHA256(sb.toString(), pubKey);
			StringBuffer querys = new StringBuffer();
			querys.append("access_token="+access_token);
			querys.append("&signature="+sign);
			querys.append("&client_id="+client_id);
			entity.setQuerys(querys.toString());
			headers.put("signature",sign);
//            headers.put("content-type","application/x-www-form-urlencoded");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return entity;
	}
}
